// TSysStat - TComponent descendant which collects the information about your system
// Has two methods:
//   procedure Open;     - Starts to collect information
//   procedure Close;    - Stops to collect information
// Has two propeties:
//   Active: Boolean;    - (Read only) Indicates the collection status (Started/Stoped)
//   SysData[Key:string]: Integer; - Gets the system data
//
// If you want to know what system values you can get and what strings you
// must pass throuth the SysData[] property, run the regedit program and go
// to the HKEY_DYN_DATA\PerfStats\StatData key and look throght the value names.
// They will be like "KERNEL\CPUUsage", "VMM\cpgFree" and so on. They are strings
// you must pass into the SysData[] property and then this property returs needed value
// (but only if you called the "Open" method before).
//
// Written by Nick Mazurkin, Russia. E-Mail: nm@yoshkar-ola.ru
// Thanks to Oleg Bulychev. Fidonet: 2:5051/28.13

unit SysStat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Registry;

type
  TSysStat = class(TComponent)
  private
    FActive   : Boolean;
    FRegistry : TRegistry;
    function    GetSysData(Key: string): Integer;
  public
    procedure   Open;
    procedure   Close;
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    property    SysData[Key: string]: Integer read GetSysData; default;
    property    Active: Boolean read FActive;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Favourites', [TSysStat]);
end;

{ TSysStat }

constructor TSysStat.Create(AOwner: TComponent);
begin
  inherited;
  FActive   := False;
  FRegistry := TRegistry.Create;
end;

destructor TSysStat.Destroy;
begin
  Close;
  FRegistry.Free;
  inherited;
end;

procedure TSysStat.Open;
var
  KeysList : TStringList;
  i        : Integer;
  Dummy    : Integer;
begin
  if (not (csDesigning in ComponentState)) and (not FActive) then
    with FRegistry do
    begin
      RootKey := HKEY_DYN_DATA;
      if OpenKey('\PerfStats\StartStat', False) then
      begin
        KeysList := TStringList.Create;
        try
          GetValueNames(KeysList);
          for i:=0 to KeysList.Count-1 do
            ReadBinaryData(KeysList.Strings[i], Dummy, 4);
        finally
         KeysList.Free;
        end;
        FActive := True;
        CloseKey;
      end;
    end;
end;

procedure TSysStat.Close;
var
  KeysList : TStringList;
  i        : Integer;
  Dummy    : Integer;
begin
  if (not (csDesigning in ComponentState)) and (FActive) then
    with FRegistry do
    begin
      RootKey := HKEY_DYN_DATA;
      if OpenKey('\PerfStats\StopStat', False) then
      begin
        KeysList := TStringList.Create;
        try
          GetValueNames(KeysList);
          for i:=0 to KeysList.Count-1 do
            ReadBinaryData(KeysList.Strings[i], Dummy, 4);
        finally
          KeysList.Free;
        end;
        FActive := False;
        CloseKey;
      end;
    end;
end;

function TSysStat.GetSysData(Key: string): Integer;
begin
  with FRegistry do
  begin
    RootKey := HKEY_DYN_DATA;
    if OpenKey('\PerfStats\StatData', False) then
    begin
      ReadBinaryData(Key, Result, 4);
      CloseKey;
    end else
      raise Exception.Create('Error opening key: '+Key);
  end;
end;

end.
